﻿#ifndef NOUN_H_INCLUDED
#define NOUN_H_INCLUDED

#include <string>
#include <map>
#include <set>
#include <vector>
#include <fstream>

#include "boost/algorithm/string.hpp"

enum NounGender
{
	NG_MALE = 0,
	NG_FEMALE,
	NG_NEUTRAL
};

struct NounRecord
{
	std::wstring nominativeForm;
	NounGender gender;

	bool haveSingleForm;
	bool haveMultipleForm;

	bool haveStandardMultipleForm;
	bool haveStandardMultipleFormWithMissingLastVowel;
	
	bool haveStandardMultipleFormEnding;
	bool haveAlternativeMultipleFormEnding;

	std::wstring specialMultipleForm;

	bool canBeAnimate;
	bool canBeInanimate;

	NounRecord();
	NounRecord(std::wstring line);
};

enum NounDeclencion
{
	FIRST_A_IFORM_INANIMATE = 0,
	FIRST_A_IFORM_ANIMATE,
	FIRST_A_UFORM_INANIMATE,
	FIRST_A_UFORM_ANIMATE,
	FIRST_YA_FORM_INANIMATE,
	FIRST_YA_FORM_ANIMATE,
	SECOND_MALE_IFORM_INANIMATE,
	SECOND_MALE_IFORM_ANIMATE,
	SECOND_MALE_UFORM_INANIMATE,
	SECOND_MALE_UFORM_ANIMATE,
	SECOND_MALE_SSFORM_INANIMATE,
	SECOND_MALE_SSFORM_ANIMATE,
	SECOND_NEUTRAL_E_FORM,
	SECOND_NEUTRAL_O_FORM,
	THIRD_FORM_INANIMATE,
	THIRD_FORM_ANIMATE,
};

enum NounGrammaticalCase
{
	NGC_P1_NOMINATIVE = 0,
	NGC_P2_GENITIVE,
	NGC_P3_DATIVE,
	NGC_P4_ACCUSATIVE,
	NGC_P5_INSTRUMENTAL,
	NGC_P6_PREPOSITIONAL,
	NGC_SIZE
};

enum NounCount
{
	NC_SINGULAR = 0,
	NC_PLURAL,
	NC_SIZE
};

struct GrammaticalTableRecord
{
	NounCount count;

	NounGrammaticalCase grammaticalCase;

	std::wstring ending;
};

struct NounDeclencionCaseTableRecord
{
	NounDeclencion nounDeclencion;
	std::vector<GrammaticalTableRecord> grammaticalCaseTable;
};



extern std::vector<NounRecord> NounRecordArr;

extern std::vector<NounDeclencionCaseTableRecord> nounDeclencionCaseTable;


NounDeclencion WStringToNounDeclencion(std::wstring str);
std::wstring NounCountToWString(NounCount nounCount);
NounCount WStringToNounCount(std::wstring str);
std::wstring NounGrammaticalCaseToWString(NounGrammaticalCase nounGrammaticalCase);
NounGrammaticalCase WStringToNounGrammaticalCase(std::wstring str);


//std::wstring NounNumberToWString(NounNumber nounNumber);

typedef std::tuple<
	NounDeclencion, 
	NounCount, 
	NounGrammaticalCase
> NounTuple;

typedef std::set<std::wstring> StringSet;

struct NounEndingDivision
{
	std::wstring base;
	std::wstring ending;

	enum DivisionCase
	{
		DC_COMMON = 0,
		DC_LOST_VOWEL_O,
		DC_LOST_VOWEL_E,
		DC_SPECIAL_PLURAL_A,
		DC_SPECIAL_PLURAL_FORM
	} divisionCase;

	bool operator<(const NounEndingDivision& other) const
	{
		if (base != other.base)
		{
			return base < other.base;
		}
		else
		{
			if (ending != other.ending)
			{
				return ending < other.ending;
			}
			else
			{
				return divisionCase < other.divisionCase;
			}
		}
	}
};

std::vector<std::wstring> GetAllNounEndingArr();

bool NounIsInDictionary(std::wstring nounNominative);
bool NounSpecialPluralFormIsInDictionary(std::wstring nounNominativePlural);


NounRecord GetNounRecordFromDictionary(std::wstring nounNominative);
NounRecord GetNounRecordFromDictionary_BySpecialPluralForm(std::wstring nounNominativePlural);

bool charIsConsolant(wchar_t c);

bool charIsVowel(wchar_t c);


struct NounStruct
{
	NounGrammaticalCase nounGrammaticalCase;
	NounCount nounCount;
	bool animated;
	NounRecord nounRecord;
};


std::set<NounEndingDivision> getPossibleNounEndingDivisionSet(std::wstring noun);

std::vector<NounTuple> GetPossibleNounTupleArr(std::wstring nounEnding);

std::wstring GetNounNoninative(std::wstring nounBase, NounDeclencion nounDeclencion, NounCount nounCount);

void SetupDeclentionMap();

bool NounFitsDeclention(NounRecord nounRecord, NounTuple nounTuple);

bool IsDeclencionSecondType(NounDeclencion nounDeclention);
bool IsDeclencionAnimated(NounDeclencion nounDeclention);

void FillDivisionCaseMaps();

std::vector<NounStruct> RecognizeNoun(std::wstring noun);

NounDeclencion CalculateNounDeclention(NounRecord nounRecord);


void LoadFrequentWordSet();
void LoadNounDeclencionCaseTable();

#endif //NOUN_H_INCLUDED
